@@ -16,6 +16,7 @@
 SM_RCSID("@(#)$Id: parseaddr.c,v 8.407 2013-11-22 20:51:56 ca Exp $")
 
 #include <sm/sendmail.h>
+#include <sm/ixlen.h>
 #include "map.h"
 
 static void	allocaddr __P((ADDRESS *, int, char *, ENVELOPE *));
@@ -42,7 +43,7 @@ static bool	hasctrlchar __P((register char *, bool, bool));
 **	of 'berkeley' -- to be transmitted over the arpanet.
 **
 **	Parameters:
-**		addr -- the address to parse.
+**		addr -- the address to parse. [i]
 **		a -- a pointer to the address descriptor buffer.
 **			If NULL, an address will be created.
 **		flags -- describe detail for parsing.  See RF_ definitions
@@ -85,6 +86,14 @@ parseaddr(addr, a, flags, delim, delimptr, e, isrcpt)
 	/*
 	**  Initialize and prescan address.
 	*/
+#if _FFR_8BITENVADDR
+	if (bitset(RF_IS_EXT, flags) && addr != NULL)
+	{
+		int len = 0;
+
+		addr = quote_internal_chars(addr, NULL, &len, NULL);
+	}
+#endif
 
 	e->e_to = addr;
 	if (tTd(20, 1))
@@ -145,6 +154,13 @@ parseaddr(addr, a, flags, delim, delimptr, e, isrcpt)
 	*/
 
 	a = buildaddr(pvp, a, flags, e);
+#if _FFR_8BITENVADDR
+	{
+		int len = 0;
+
+		a->q_user = quote_internal_chars(a->q_user, NULL, &len, e->e_rpool); /* EAI: ok */
+	}
+#endif
 
 	if (hasctrlchar(a->q_user, isrcpt, true))
 	{
@@ -239,6 +255,7 @@ parseaddr(addr, a, flags, delim, delimptr, e, isrcpt)
 
 	return a;
 }
+
 /*
 **  INVALIDADDR -- check for address containing characters used for macros
 **
@@ -264,7 +281,7 @@ invalidaddr(addr, delimptr, isrcpt)
 	bool result = false;
 	char savedelim = '\0';
 	char *b = addr;
-	int len = 0;
+	XLENDECL
 
 	if (delimptr != NULL)
 	{
@@ -275,24 +292,30 @@ invalidaddr(addr, delimptr, isrcpt)
 	}
 	for (; *addr != '\0'; addr++)
 	{
+#if !USE_EAI
 		if (!EightBitAddrOK && (*addr & 0340) == 0200)
 		{
 			setstat(EX_USAGE);
 			result = true;
 			*addr = BAD_CHAR_REPLACEMENT;
 		}
-		if (++len > MAXNAME - 1)
+#endif
+		XLEN(*addr);
+		if (xlen > MAXNAME - 1)	/* EAI:ok */
 		{
 			char saved = *addr;
 
 			*addr = '\0';
 			usrerr("553 5.1.0 Address \"%s\" too long (%d bytes max)",
-			       b, MAXNAME - 1);
+			       b, MAXNAME - 1);	/* EAI:ok */
 			*addr = saved;
 			result = true;
 			goto delim;
 		}
 	}
+#if USE_EAI
+	/* check for valid UTF8 string? */
+#endif
 	if (result)
 	{
 		if (isrcpt)
@@ -307,6 +330,7 @@ delim:
 		*delimptr = savedelim;	/* restore old character at delimptr */
 	return result;
 }
+
 /*
 **  HASCTRLCHAR -- check for address containing meta-characters
 **
@@ -325,6 +349,9 @@ delim:
 **		true -- if the address has any "weird" characters or
 **			non-printable characters or if a quote is unbalanced.
 **		false -- otherwise.
+**
+**	Side Effects:
+**		Might invoke shorten_rfc822_string() to change addr in place.
 */
 
 static bool
@@ -333,26 +360,27 @@ hasctrlchar(addr, isrcpt, complain)
 	bool isrcpt, complain;
 {
 	bool quoted = false;
-	int len = 0;
 	char *result = NULL;
 	char *b = addr;
+	XLENDECL
 
 	if (addr == NULL)
 		return false;
 	for (; *addr != '\0'; addr++)
 	{
-		if (++len > MAXNAME - 1)
+		XLEN(*addr);
+		if (xlen > MAXNAME - 1)	/* EAI:ok */
 		{
 			if (complain)
 			{
-				(void) shorten_rfc822_string(b, MAXNAME - 1);
+				(void) shorten_rfc822_string(b, MAXNAME - 1);	/* EAI:ok */
 				usrerr("553 5.1.0 Address \"%s\" too long (%d bytes max)",
-				       b, MAXNAME - 1);
+				       b, MAXNAME - 1);	/* EAI:ok */
 				return true;
 			}
 			result = "too long";
 		}
-		if (!EightBitAddrOK && !quoted && (*addr < 32 || *addr == 127))
+		if (!quoted && ((unsigned char)*addr < 32 || *addr == 127))
 		{
 			result = "non-printable character";
 			*addr = BAD_CHAR_REPLACEMENT;
@@ -370,7 +398,7 @@ hasctrlchar(addr, isrcpt, complain)
 				break;
 			}
 		}
-		if (!EightBitAddrOK && (*addr & 0340) == 0200)
+		if (!SMTPUTF8 && !EightBitAddrOK && (*addr & 0340) == 0200)
 		{
 			setstat(EX_USAGE);
 			result = "8-bit character";
@@ -391,6 +419,7 @@ hasctrlchar(addr, isrcpt, complain)
 	}
 	return result != NULL;
 }
+
 /*
 **  ALLOCADDR -- do local allocations of address on demand.
 **
@@ -686,6 +715,7 @@ prescan(addr, delim, pvpbuf, pvpbsize, delimptr, toktab, ignore)
 	char *saveto = CurEnv->e_to;
 	static char *av[MAXATOM + 1];
 	static bool firsttime = true;
+	XLENDECL
 
 	if (firsttime)
 	{
@@ -740,6 +770,7 @@ prescan(addr, delim, pvpbuf, pvpbsize, delimptr, toktab, ignore)
 	{
 		/* read a token */
 		tok = q;
+		XLENRESET;
 		for (;;)
 		{
 			/* store away any old lookahead character */
@@ -750,6 +781,8 @@ prescan(addr, delim, pvpbuf, pvpbsize, delimptr, toktab, ignore)
 				{
 	addrtoolong:
 					usrerr("553 5.1.1 Address too long");
+
+					/* ilenx()? */
 					if (strlen(addr) > MAXNAME)
 						addr[MAXNAME] = '\0';
 	returnnull:
@@ -760,6 +793,8 @@ prescan(addr, delim, pvpbuf, pvpbsize, delimptr, toktab, ignore)
 						*delimptr = p;
 					}
 					CurEnv->e_to = saveto;
+					if (tTd(22, 12))
+						sm_dprintf("prescan: ==> NULL\n");
 					return NULL;
 				}
 
@@ -769,6 +804,7 @@ prescan(addr, delim, pvpbuf, pvpbsize, delimptr, toktab, ignore)
 				    !EightBitAddrOK)
 					c &= 0x7f;
 #endif /* !ALLOW_255 */
+				XLEN(c);
 				*q++ = c;
 			}
 
@@ -834,6 +870,7 @@ prescan(addr, delim, pvpbuf, pvpbsize, delimptr, toktab, ignore)
 					if (q >= &pvpbuf[pvpbsize - 5])
 						goto addrtoolong;
 					*q++ = '\\';
+					XLEN('\\');
 					continue;
 				}
 			}
@@ -874,7 +911,7 @@ prescan(addr, delim, pvpbuf, pvpbsize, delimptr, toktab, ignore)
 				char *ptr = p;
 
 				anglecnt++;
-				while (isascii(*ptr) && isspace(*ptr))
+				while (SM_ISSPACE(*ptr))
 					ptr++;
 				if (*ptr == '@')
 					route_syntax = true;
@@ -893,7 +930,7 @@ prescan(addr, delim, pvpbuf, pvpbsize, delimptr, toktab, ignore)
 					anglecnt--;
 				route_syntax = false;
 			}
-			else if (delim == ' ' && isascii(c) && isspace(c))
+			else if (delim == ' ' && SM_ISSPACE(c))
 				c = ' ';
 
 			if (c == NOCHAR)
@@ -939,7 +976,7 @@ prescan(addr, delim, pvpbuf, pvpbsize, delimptr, toktab, ignore)
 				usrerr("553 5.1.0 prescan: too many tokens");
 				goto returnnull;
 			}
-			if (q - tok > MAXNAME)
+			if (xlen > MAXNAME)	/* EAI:ok */
 			{
 				usrerr("553 5.1.0 prescan: token too long");
 				goto returnnull;
@@ -991,7 +1028,7 @@ prescan(addr, delim, pvpbuf, pvpbsize, delimptr, toktab, ignore)
 **	and try over again.
 **
 **	Parameters:
-**		pvp -- pointer to token vector.
+**		pvp -- pointer to token vector. [i]
 **		ruleset -- the ruleset to use for rewriting.
 **		reclevel -- recursion level (to catch loops).
 **		e -- the current envelope.
@@ -1237,7 +1274,7 @@ rewrite(pvp, ruleset, reclevel, e, maxatom)
 
 			  default:
 				/* must have exact match */
-				if (sm_strcasecmp(rp, ap))
+				if (!SM_STRCASEEQ(rp, ap))
 					goto backup;
 				avp++;
 				break;
@@ -1356,7 +1393,7 @@ rewrite(pvp, ruleset, reclevel, e, maxatom)
 					pp = m->match_first;
 					while (pp <= m->match_last)
 					{
-						sm_dprintf(" %p=\"", *pp);
+						sm_dprintf(" %p=\"", (void *)*pp);
 						sm_dflush();
 						sm_dprintf("%s\"", *pp++);
 					}
@@ -1402,7 +1439,7 @@ rewrite(pvp, ruleset, reclevel, e, maxatom)
 						sm_dprintf("rewrite: RHS $&{%s} => \"%s\"\n",
 							macname(rp[1]),
 							mval == NULL ? "(NULL)" : mval);
-					if (mval == NULL || *mval == '\0')
+					if (SM_IS_EMPTY(mval))
 						continue;
 
 					/* save the remainder of the input */
@@ -1539,7 +1576,7 @@ rewrite(pvp, ruleset, reclevel, e, maxatom)
 				{
 					cataddr(xpvp, NULL, replac,
 						&pvpbuf[sizeof(pvpbuf)] - replac,
-						'\0', false);
+						'\0', true);
 					if (arg_rvp <
 					    &argvect[MAX_MAP_ARGS - 1])
 						*++arg_rvp = replac;
@@ -1563,7 +1600,7 @@ rewrite(pvp, ruleset, reclevel, e, maxatom)
 			{
 				cataddr(xpvp, NULL, replac,
 					&pvpbuf[sizeof(pvpbuf)] - replac,
-					'\0', false);
+					'\0', true);
 				if (arg_rvp < &argvect[MAX_MAP_ARGS - 1])
 					*++arg_rvp = replac;
 			}
@@ -1666,6 +1703,7 @@ rewrite(pvp, ruleset, reclevel, e, maxatom)
 	}
 	return rstat;
 }
+
 /*
 **  CALLSUBR -- call subroutines in rewrite vector
 **
@@ -1782,8 +1820,8 @@ callsubr(pvp, reclevel, e)
 **
 **	Parameters:
 **		smap -- the map to use for the lookup.
-**		key -- the key to look up.
-**		argvect -- arguments to pass to the map lookup.
+**		key -- the key to look up. [x]
+**		argvect -- arguments to pass to the map lookup. [x]
 **		pstat -- a pointer to an integer in which to store the
 **			status from the lookup.
 **		e -- the current envelope.
@@ -1975,8 +2013,8 @@ buildaddr(tv, a, flags, e)
 	register char *p;
 	char *mname;
 	char **hostp;
-	char hbuf[MAXNAME + 1];
-	static char ubuf[MAXNAME + 2];
+	char hbuf[MAXNAME + 1];	/* EAI:ok */
+	static char ubuf[MAXNAME_I + 2];
 
 	if (tTd(24, 5))
 	{
@@ -2043,12 +2081,12 @@ badaddr:
 	if (tv == hostp)
 		hostp = NULL;
 	else if (hostp != NULL)
-		cataddr(hostp, tv - 1, hbuf, sizeof(hbuf), '\0', false);
+		cataddr(hostp, tv - 1, hbuf, sizeof(hbuf), '\0', true);
 	cataddr(++tv, NULL, ubuf, sizeof(ubuf), ' ', false);
 	--maxatom;
 
 	/* save away the host name */
-	if (sm_strcasecmp(mname, "error") == 0)
+	if (SM_STRCASEEQ(mname, "error"))
 	{
 		/* Set up triplet for use by -bv */
 		a->q_mailer = &errormailer;
@@ -2073,7 +2111,7 @@ badaddr:
 			else
 			{
 				for (ep = ErrorCodes; ep->ec_name != NULL; ep++)
-					if (sm_strcasecmp(ep->ec_name, hbuf) == 0)
+					if (SM_STRCASEEQ(ep->ec_name, hbuf))
 						break;
 				setstat(ep->ec_code);
 			}
@@ -2119,7 +2157,7 @@ badaddr:
 
 	for (mp = Mailer; (m = *mp++) != NULL; )
 	{
-		if (sm_strcasecmp(m->m_name, mname) == 0)
+		if (SM_STRCASEEQ(m->m_name, mname))
 			break;
 	}
 	if (m == NULL)
@@ -2194,9 +2232,9 @@ badaddr:
 	*/
 
 	if (a->q_host != NULL && !bitnset(M_HST_UPPER, m->m_flags))
-		makelower(a->q_host);
+		makelower_a(&a->q_host, e->e_rpool);
 	if (!bitnset(M_USR_UPPER, m->m_flags))
-		makelower(a->q_user);
+		makelower_a(&a->q_user, e->e_rpool);
 
 	if (tTd(24, 6))
 	{
@@ -2210,7 +2248,7 @@ badaddr:
 **  CATADDR -- concatenate pieces of addresses (putting in <LWSP> subs)
 **
 **	Parameters:
-**		pvp -- parameter vector to rebuild.
+**		pvp -- parameter vector to rebuild. [i]
 **		evp -- last parameter to include.  Can be NULL to
 **			use entire pvp.
 **		buf -- buffer to build the string into.
@@ -2219,7 +2257,7 @@ badaddr:
 **			'\0': SpaceSub.
 **			NOSPACESEP: no separator
 **		external -- convert to external form?
-**			(no metacharacters; METAQUOTEs removed, see below)
+**			(undo "meta quoting")
 **
 **	Returns:
 **		none.
@@ -2238,9 +2276,6 @@ badaddr:
 **	The cataddr routine needs to be aware of whether it is producing
 **	an internal or external form as output (it only takes internal
 **	form as input).
-**
-**	The parseaddr routine has a similar issue on input, but that
-**	is flagged on the basis of which token table is passed in.
 */
 
 void
@@ -2298,8 +2333,8 @@ cataddr(pvp, evp, buf, sz, spacesub, external)
 			**  If the current character (c) is METAQUOTE and we
 			**  want the "external" form and the next character
 			**  is not NUL, then overwrite METAQUOTE with that
-			**  character (i.e., METAQUOTE ch is changed to
-			**  ch).  p[-1] is used because p is advanced (above).
+			**  character (i.e., METAQUOTE ch is changed to ch).
+			**  p[-1] is used because p is advanced (above).
 			*/
 
 			if ((c & 0377) == METAQUOTE && external && *q != '\0')
@@ -2449,8 +2484,10 @@ static struct qflags	AddressFlags[] =
 	{ "QBYNRELAY",		QBYNRELAY	},
 	{ "QINTBCC",		QINTBCC		},
 	{ "QDYNMAILER",		QDYNMAILER	},
-	{ "QRCPTOK",		QRCPTOK		},
 	{ "QSECURE",		QSECURE		},
+	{ "QTHISPASS",		QTHISPASS	},
+	{ "QRCPTOK",		QRCPTOK		},
+	{ "QQUEUED",		QQUEUED		},
 	{ NULL,			0		}
 };
 
@@ -2473,7 +2510,7 @@ printaddr(fp, a, follow)
 
 	while (a != NULL)
 	{
-		(void) sm_io_fprintf(fp, SM_TIME_DEFAULT, "%p=", a);
+		(void) sm_io_fprintf(fp, SM_TIME_DEFAULT, "%p=", (void *)a);
 		(void) sm_io_flush(fp, SM_TIME_DEFAULT);
 
 		/* find the mailer -- carefully */
@@ -2576,7 +2613,7 @@ printaddr(fp, a, follow)
 		}
 		(void) sm_io_fprintf(fp, SM_TIME_DEFAULT,
 				     ", next=%p, alias %p, uid %d, gid %d\n",
-				     a->q_next, a->q_alias,
+				     (void *)a->q_next, (void *)a->q_alias,
 				     (int) a->q_uid, (int) a->q_gid);
 		(void) sm_io_fprintf(fp, SM_TIME_DEFAULT, "\tflags=%lx<",
 				     a->q_flags);
@@ -2637,11 +2674,12 @@ emptyaddr(a)
 	return a->q_paddr == NULL || strcmp(a->q_paddr, "<>") == 0 ||
 	       a->q_user == NULL || strcmp(a->q_user, "<>") == 0;
 }
+
 /*
 **  REMOTENAME -- return the name relative to the current mailer
 **
 **	Parameters:
-**		name -- the name to translate.
+**		name -- the name to translate. [i]
 **		m -- the mailer that we want to do rewriting relative to.
 **		flags -- fine tune operations.
 **		pstat -- pointer to status word.
@@ -2649,10 +2687,7 @@ emptyaddr(a)
 **
 **	Returns:
 **		the text string representing this address relative to
-**			the receiving mailer.
-**
-**	Side Effects:
-**		none.
+**			the receiving mailer. [i]
 **
 **	Warnings:
 **		The text string returned is tucked away locally;
@@ -2671,8 +2706,8 @@ remotename(name, m, flags, pstat, e)
 	char *SM_NONVOLATILE fancy;
 	char *oldg;
 	int rwset;
-	static char buf[MAXNAME + 1];
-	char lbuf[MAXNAME + 1];
+	static char buf[MAXNAME_I + 1];
+	char lbuf[MAXNAME_I + 1];
 	char pvpbuf[PSBUFSIZE];
 	char addrtype[4];
 
@@ -2943,7 +2978,7 @@ dequote_init(map, args)
 	map->map_mflags |= MF_KEEPQUOTES;
 	for (;;)
 	{
-		while (isascii(*p) && isspace(*p))
+		while (SM_ISSPACE(*p))
 			p++;
 		if (*p != '-')
 			break;
@@ -2962,7 +2997,7 @@ dequote_init(map, args)
 			map->map_spacesub = *++p;
 			break;
 		}
-		while (*p != '\0' && !(isascii(*p) && isspace(*p)))
+		while (*p != '\0' && !(SM_ISSPACE(*p)))
 			p++;
 		if (*p != '\0')
 			*p = '\0';
@@ -3076,7 +3111,7 @@ dequote_map(map, name, av, statp)
 **	Parameters:
 **		rwset -- the rewriting set to use.
 **		p1 -- the first string to check.
-**		p2 -- the second string to check -- may be null.
+**		p2 -- the second string to check -- may be NULL.
 **		e -- the current envelope.
 **		flags -- control some behavior, see RSF_ in sendmail.h
 **		logl -- logging level.
@@ -3089,6 +3124,7 @@ dequote_map(map, name, av, statp)
 **
 **	Returns:
 **		EX_OK -- if the rwset doesn't resolve to $#error
+**			or is not defined
 **		else -- the failure status (message printed)
 */
 
@@ -3315,7 +3351,7 @@ rscheck(rwset, p1, p2, e, flags, logl, host, logid, addr, addrstr)
 **	Parameters:
 **		rwset -- the rewriting set to use.
 **		p1 -- the first string to check.
-**		p2 -- the second string to check -- may be null.
+**		p2 -- the second string to check -- may be NULL.
 **		e -- the current envelope.
 **		pvp -- pointer to token vector.
 **		pvpbuf -- buffer space.
